scriptDir = 'scripts'
source(file.path(scriptDir, 'setup.R'))

params = fread(file.path(paramDir, 'analysis_params.csv'))
measDrugExp = fread(file.path(dataDir, 'screen_conf_drug_exp.csv.gz'))
measScreenConf = fread(file.path(dataDir, 'screen_conf.csv.gz'))

statsPosNeg = fread(file.path(resultDir, 'stats_pos_neg.csv'))
statsPosPos = fread(file.path(resultDir, 'stats_pos_pos.csv'))

assayMetadata = fread(file.path(paramDir, 'assay_metadata.csv'))
ingredMetadata = fread(file.path(paramDir, 'ingred_metadata.csv'))

spikingMetadata = fread(file.path(dataDir, 'spiking_metadata.csv'))

########################################

dCountsPersons = measScreenConf[, .(n_persons = uniqueN(person_id))]
fwrite(dCountsPersons, file.path(resultDir, 'counts_persons.csv'))

dCountsMeas = measScreenConf[, .(n_meas = .N),
                             keyby = .(screen_target, measurement_result)]
dCountsMeas = dcast(dCountsMeas, screen_target ~ measurement_result,
                    value.var = 'n_meas')
fwrite(dCountsMeas, file.path(resultDir, 'counts_meas.csv'))

dCountsIngreds = merge(measDrugExp[, .(n_ingreds = uniqueN(ingred_name)),
                                    by = .(person_id, measurement_datetime)],
                       measScreenConf, by = c('person_id', 'measurement_datetime'),
                       all.y = TRUE)
dCountsIngreds[is.na(n_ingreds), n_ingreds := 0]
fwrite(as.list(summary(dCountsIngreds$n_ingreds)),
       file.path(resultDir, 'counts_ingreds.csv'))

########################################

screenIds = sort(unique(statsPosNeg$screen_id))

plotList = foreach(screenId = screenIds) %do% {
  statsNow = statsPosNeg[screen_id == screenId][1:min(params$n_top, .N)]
  d = getWideData(measDrugExp[measurement_result != 'pos_pos'],
                  measScreenConf, screenId, statsNow$ingred_name)
  coexpMat = getCoexpFreq(d, statsNow$ingred_name) * 100
  plotCoexpFreq(coexpMat, statsNow, screenId, screenId, resultDir,
                annotateIngredCategory = FALSE)[[4]]}

names(plotList) = screenIds

########################################

orDummyTp = 2^(-5)
orDummyFp = 2^(-3)

dOr = merge(statsPosNeg[, .(screen_id, screen_target, ingred_name, or_fp = or_pm)],
            statsPosPos[, .(screen_id, screen_target, ingred_name, or_tp = or_pm)],
            by = c('screen_id', 'screen_target', 'ingred_name'), all = TRUE)

dOr[is.na(or_tp), or_tp := orDummyTp]
dOr[is.na(or_fp), or_fp := orDummyFp]

dOr = merge(dOr, ingredMetadata[ingred_category == 'assay target' |
                                  !is.na(insert_conc_positive_ng_ml),
                                .(screen_id, ingred_name, ingred_category)],
            by = c('screen_id', 'ingred_name'), all.x = TRUE)
dOr[is.na(ingred_category), ingred_category := 'neither']

# dOr = merge(dOr, assayMetadata[assay_type == 'screen',
#                                .(screen_id = assay_id, screen_target)],
#             by = 'screen_id')

dOr[screen_id != 'MDMA', screen_target := str_to_sentence(screen_target)]
setorder(dOr, -ingred_category) # so targets and interferents are plotted on top

########################################

xLimBreaks = seq(log2(orDummyFp), 10, 2)
xLimLabels = as.character(xLimBreaks)
xLimLabels[1] = 'NA'

yLimBreaks = seq(log2(orDummyTp), 10, 2)
yLimLabels = as.character(yLimBreaks)
yLimLabels[1] = 'NA'

pPre = ggplot(dOr) +
  facet_wrap(~ screen_target, ncol = 4, scales = 'free') +
  geom_hline(yintercept = 0, color = 'gray') + #, size = 0.4) +
  geom_vline(xintercept = 0, color = 'gray') + #, size = 0.4) +
  geom_point(aes(x = log2(or_fp), y = log2(or_tp), fill = ingred_category,
                 color = ingred_category, shape = ingred_category,
                 alpha = ingred_category, size = ingred_category), stroke = 0) +
  labs(x = expression('log'[2]*'(odds ratio for false positives)'),
       y = expression('log'[2]*'(odds ratio for true positives)'),
       fill = 'Ingredient category', color = 'Ingredient category',
       shape = 'Ingredient category', alpha = 'Ingredient category',
       size = 'Ingredient category') +
  scale_x_continuous(breaks = xLimBreaks, labels = xLimLabels) +
  scale_y_continuous(breaks = yLimBreaks, labels = yLimLabels) +
  scale_fill_manual(values = c('#e41a1c', '#377eb8', 'black')) +
  scale_color_manual(values = c('#e41a1c', '#377eb8', 'black')) +
  scale_shape_manual(values = c(22, 23, 21),
                     guide = guide_legend(override.aes = list(size = 2))) +
  scale_alpha_manual(values = c(0.5, 0.5, 0.2)) +
  scale_size_manual(values = c(1.7, 1.7, 1.3), guide = FALSE)

p = reposition_legend(pPre, 'left', panel = 'panel-4-3', plot = FALSE)

ggsave(file.path(resultDir, 'stats_odds_ratios.pdf'),
       plot = p, width = 6.75, height = 5.5)

########################################

ingredGroups = data.table(group_id = 1, screen_id = 'Cannab',
                          ingred_name = c('meloxicam', 'atorvastatin', 'furosemide'))

ingredGroupsUnique = unique(ingredGroups[, .(group_id, screen_id)])

statsGroups = foreach(i = 1:nrow(ingredGroupsUnique), .combine = rbind) %dopar% {
  ingredNames = ingredGroups[ingredGroupsUnique[i], on = c('group_id', 'screen_id'),
                             nomatch = NULL]$ingred_name

  d = getWideData(measDrugExp[measurement_result != 'pos_pos'], measScreenConf,
                  ingredGroupsUnique$screen_id[i], ingredNames)
  d[, y := 0]
  d[measurement_result == 'pos_neg', y := 1]

  modelStr = sprintf('y ~ `%s`', paste(ingredNames, collapse = '`+`'))
  statsNow = getStats(d, modelStr = modelStr)
  cbind(ingredGroupsUnique[i], statsNow)}

fwrite(statsGroups, file.path(resultDir, 'stats_pos_neg_groups.csv'))

########################################

statsSpiking = merge(statsPosNeg, spikingMetadata, by = colnames(spikingMetadata))
fwrite(statsSpiking, file.path(resultDir, 'stats_spiking.csv'))
