library('ashr')
library('cba')
library('cowplot')
library('data.table')
library('doParallel')
library('ggplot2')
library('ggstance')
library('lemon')
library('logistf')
library('pheatmap')
library('RODBC')
library('stringr')

theme_set(theme_classic() +
            theme(axis.text = element_text(color = 'black'),
                  legend.margin = margin(t = 0, r = 0, b = 0, l = 0, unit = 'cm')))

set.seed(2)
registerDoParallel()
paramDir = 'params'
dataDir = 'data'
resultDir = 'results'


sqlDrop = function(con, tableName) {
  sqlQuery(con, sprintf('drop table %s if exists', tableName))}


sqlDropSave = function(con, tableName, dt) {
  # prevent netezza from choking on lowercase column names and table names
  d = copy(dt)
  setnames(d, toupper(colnames(d)))
  sqlDrop(con, tableName)
  sqlSave(con, d, rownames = FALSE, safer = FALSE,
          tablename = toupper(tableName))}


sqlDropQuery = function(con, tableName, q) {
  sqlDrop(con, tableName)
  sqlQuery(con, q)}


sqlFetchDt = function(con, tableName) {
  d = sqlFetch(con, toupper(tableName), stringsAsFactors = FALSE, as.is = TRUE)
  setDT(d)
  setnames(d, tolower(colnames(d)))
  setorder(d)
  return(d)}


getWideData = function(measDrugExp, measScreenConf, screenId = NULL,
                       ingredNames = NULL) {
  idCols = c('screen_id', 'person_id', 'measurement_datetime', 'measurement_result')
  castFormula = sprintf('%s ~ %s', paste(idCols, collapse = '+'), 'ingred_name')

  if (is.null(screenId)) {
    m1 = copy(measDrugExp)
    m2 = measScreenConf[, ..idCols]
  } else {
    m1 = measDrugExp[screen_id == screenId]
    m2 = measScreenConf[screen_id == screenId, ..idCols]}

  if (is.null(ingredNames)) {
    ingredNames = unique(m1$ingred_name)
  } else {
    m1 = m1[ingred_name %in% ingredNames]}
  m1[, value := 1]
  m3 = dcast(m1, castFormula, value.var = 'value', fill = 0)
  m4 = merge(m2, m3, all.x = TRUE, by = idCols)

  for (ingredName in ingredNames) {
    set(m4, which(is.na(m4[[ingredName]])), ingredName, 0)}
  return(m4[])}


getStats = function(d, ingredName, yName = 'y', modelStr = NULL) {
  if (is.null(modelStr)) {
    formStr = sprintf('`%s` ~ `%s`', yName, ingredName)
  } else {
    formStr = modelStr}
  fit = logistf(formula(formStr), data = d)
  s = data.table(names(fit$coefficients[-1]), fit$coefficients[-1],
                 sqrt(diag(fit$var)[-1]), fit$prob[-1])
  setnames(s, c('term', 'coef', 'se', 'pval'))
  # s[, or := exp(coef)]
  # s[, or_ci_lo := exp(fit$ci.lower[-1])]
  # s[, or_ci_up := exp(fit$ci.upper[-1])]
  return(s[])}


getMeasDrugExpStats = function(measDrugExp, measScreenConf, minPersonsPos,
                               measResult1 = 'neg', measResult2 = 'pos_neg') {
  idCols = c('screen_id', 'person_id', 'measurement_datetime', 'measurement_result')
  castFormula = sprintf('%s ~ %s', paste(idCols, collapse = '+'), 'ingred_name')

  m1 = measDrugExp[measurement_result %in% c(measResult1, measResult2)]
  m1[, value := 1]
  m2 = measScreenConf[measurement_result %in% c(measResult1, measResult2), ..idCols]

  # unusual ordering of do and dopar based on number of cores and number of screens
  s = foreach(screenId = sort(unique(m2$screen_id)), .combine = rbind) %do% {
    mNow = m1[screen_id == screenId,
              if (uniqueN(person_id[measurement_result == measResult2]) >= minPersonsPos) .SD,
              by = .(screen_id, ingred_name)]

    if (nrow(mNow) == 0) {
      return(data.table())}

    wideData = getWideData(mNow, m2[screen_id == screenId])
    ingredNames = setdiff(colnames(wideData), idCols)
    wideData[, y := ifelse(measurement_result == measResult1, 0, 1)]

    sScreen = foreach(ingredName = ingredNames, .combine = rbind) %dopar% {
      getStats(wideData, ingredName)}
    sScreen[, ingred_name := ingredNames]

    sPre = mNow[, .(exposure_freq = .N / nrow(wideData)),
                by = .(screen_id, ingred_name)]
    sScreen = merge(sPre, sScreen, by = 'ingred_name')

    setcolorder(sScreen, 'screen_id')}
    # setorder(sScreen, -or_ci_lo)}
  return(s[, term := NULL][])}


getAshStats = function(dStats, mixcompdist = 'halfuniform', ...) {
  ashObj = ash(dStats$coef, dStats$se, mixcompdist = mixcompdist, ...)
  dAsh = setDT(ashObj$result)[, !c('betahat', 'sebetahat')]
  ci = ashci(ashObj)
  dAsh[, `:=`(credint_lower = ci[, 1L], credint_upper = ci[, 2L])]
  return(dAsh[])}


getOrderedMat = function(mat) {
  if (nrow(mat) < 3) {
    return(mat)}
  dd = dist(mat)
  co = order.optimal(dd, hclust(dd)$merge)
  mat = mat[co$order, ]
  mat = mat[, co$order]
  return(mat)}


getCoexpFreq = function(wideData, ingredNames) {
  # fraction of exposures to ingred in row,
  # in which person also exposed to ingred in col
  d = copy(wideData)
  d[, idx := 1:.N]
  d = melt(d, id.vars = 'idx', measure.vars = ingredNames, variable.name = 'ingred')

  d1 = merge(d[, .(idx, ingred1 = ingred, val1 = value)],
             d[, .(idx, ingred2 = ingred, val2 = value)],
             by = 'idx', allow.cartesian = TRUE)
  d2 = d1[, .(v = sum(val1 * val2) / sum(val1)), by = .(ingred1, ingred2)]
  d3 = dcast(d2, ingred1 ~ ingred2, value.var = 'v')

  mat = as.matrix(d3[, 2:ncol(d3)])
  rownames(mat) = colnames(mat)

  mat = getOrderedMat(mat)
  diag(mat) = NA
  return(mat)}


plotCoexpFreq = function(coexpMat, measDrugExpStats, screenId, main,
                         resultDir, palette = 'Greys',
                         annotateIngredCategory = TRUE, maxChars = 20,
                         width = 6, height = 4.5) {

  drugAnnoColors = list(`log2(OR)` = scales::brewer_pal('seq', palette)(5))

  if (annotateIngredCategory) {
    drugAnnoPre = measDrugExpStats[, .(ingred_name, `log2(OR)` = log2(or_pm), Category)]
    drugAnnoColors = c(list(Category = c('assay target' = '#66c2a5',
                                         'known cross-reactant' = '#fc8d62',
                                         'neither' = 'white')),
                       drugAnnoColors)
  } else {
    drugAnnoPre = measDrugExpStats[, .(ingred_name, `log2(OR)` = log2(or_pm))]}

  drugAnnoPre[, `log2(OR)` := round(`log2(OR)`, 2)]
  drugAnno = data.frame(ingred_name = rownames(coexpMat), stringsAsFactors = FALSE)
  drugAnno = merge(drugAnno, drugAnnoPre, by = 'ingred_name', sort = FALSE)
  rownames(drugAnno) = rownames(coexpMat)
  drugAnno$ingred_name = NULL

  drugsShort = ifelse(nchar(rownames(coexpMat)) <= maxChars, rownames(coexpMat),
                      paste0(substr(rownames(coexpMat), 1, maxChars - 2), '...'))

  pheatmap(coexpMat, color = colorRampPalette(RColorBrewer::brewer.pal(7, 'YlGnBu'))(100),
           cluster_rows = FALSE, cluster_cols = FALSE,
           annotation_row = drugAnno, annotation_col = drugAnno,
           annotation_colors = drugAnnoColors,
           labels_row = drugsShort, labels_col = drugsShort, main = main,
           filename = file.path(resultDir, sprintf('coexp_%s.pdf', screenId)),
           width = width, height = height, silent = TRUE)}
